@@ -5,7 +5,7 @@ use warnings;
 
 use mop::internals::util;
 
-our $VERSION   = '0.02';
+our $VERSION   = '0.03';
 our $AUTHORITY = 'cpan:STEVAN';
 
 use parent 'mop::role';
@@ -59,7 +59,12 @@ sub BUILD {
     }
 }
 
-sub create_fresh_instance_structure { (shift)->instance_generator->() }
+sub new_fresh_instance {
+    my $self = shift;
+    my $instance = bless $self->instance_generator->(), $self->name;
+    mop::internals::util::register_object($instance);
+    return $instance;
+}
 
 sub new_instance {
     my $self = shift;
@@ -68,13 +73,15 @@ sub new_instance {
     die 'Cannot instantiate abstract class (' . $self->name . ')'
         if $self->is_abstract;
 
-    my $instance = bless $self->create_fresh_instance_structure, $self->name;
-    mop::internals::util::register_object($instance);
+    my $instance = $self->new_fresh_instance;
 
     my %attributes = map {
         if (my $m = mop::meta($_)) {
             %{ $m->attribute_map }
         }
+        else {
+            ()
+        }
     } reverse @{ mro::get_linear_isa($self->name) };
 
     foreach my $attr (values %attributes) {
@@ -149,9 +156,9 @@ sub __INIT_METACLASS__ {
     $METACLASS->add_method( mop::method->new( name => 'is_abstract',         body => \&is_abstract         ) );
     $METACLASS->add_method( mop::method->new( name => 'make_class_abstract', body => \&make_class_abstract ) );
 
-    $METACLASS->add_method( mop::method->new( name => 'instance_generator',              body => \&instance_generator              ) );
-    $METACLASS->add_method( mop::method->new( name => 'set_instance_generator',          body => \&set_instance_generator          ) );
-    $METACLASS->add_method( mop::method->new( name => 'create_fresh_instance_structure', body => \&create_fresh_instance_structure ) );
+    $METACLASS->add_method( mop::method->new( name => 'instance_generator',     body => \&instance_generator     ) );
+    $METACLASS->add_method( mop::method->new( name => 'set_instance_generator', body => \&set_instance_generator ) );
+    $METACLASS->add_method( mop::method->new( name => 'new_fresh_instance',     body => \&new_fresh_instance     ) );
 
     $METACLASS->add_method( mop::method->new( name => 'new_instance',   body => \&new_instance   ) );
     $METACLASS->add_method( mop::method->new( name => 'clone_instance', body => \&clone_instance ) );
@@ -189,7 +196,7 @@ TODO
 
 =item C<set_instance_generator($generator)>
 
-=item C<create_fresh_instance_structure>
+=item C<new_fresh_instance>
 
 =item C<new_instance(%args)>
 
@@ -221,7 +228,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2013 by Infinity Interactive.
+This software is copyright (c) 2013-2014 by Infinity Interactive.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
